# Alterator::Backend3 module

raise "You may wish to require 'alterator' instead of 'alterator/backend3'." unless defined? Alterator::NoActionError

module Alterator

  class Backend3

    def initialize (src = $stdin, dst = $stdout, err = $stderr)
      @src = src
      @dst = dst.clone
      dst.reopen(err)
      dst.sync = true
      @err = err

      @message_block = nil
      @reply = ""

      @src.sync = true
      @dst.sync = true
      @err.sync = true
    end

    def write_plain(wrap, *arg)
      @reply << "(" if wrap
      @reply << arg.join(" ")
      @reply << ")" if wrap
    end
    private :write_plain

    def write(*obj)
      case obj.length
      when 0
        write(nil)
      when 1
        if obj[0].nil?
          # nop
        elsif obj[0].respond_to? :each_pair
          obj[0].each_pair do |k,v|
            write_plain(true, "name", k.to_s.quote, "label", (v.to_s.empty? ? k : v).to_s.quote)
          end
        else
          raise "Backend erorr: dunno how to write \"#{obj[0].inspect}\" of class #{obj[0].class}"
        end
      when 2
        case obj[0]
        when NilClass
          write_plain(false, obj[1].quote)
        when Symbol
          write_plain(false, obj[0].to_s, obj[1].quote)
        when String
          write_plain(true, obj[0].quote, obj[1].quote(false))
        else
        raise "Backend erorr: dunno how to write object with name \"#{obj[0].inspect}\" of class #{obj[0].class}"
        end
      else
        raise "Backend erorr: dunno how to write \"#{obj.inspect}\""
      end
      @reply << "\n"
    end

    def on_message (&block)
      @message_block = block
    end

    def message_loop ()
      message = Hash.new
      reading = false
      @src.each_line do |line|
        line.chomp!
        if line == "_message:begin"
          message = Hash.new
          reading = true
        elsif reading and line == "_message:end"
          reading = false
          error = false
          @reply = ""
          begin
            if block_given?
              yield message
            else
              @message_block.call(message)
            end
          rescue NoActionError
            error = true
          rescue Exception => e
            @reply = ""
            write(:error, e)
          end
          if error
            @dst << "#f\n"
          else
            @dst << "(\n"
            @dst << @reply
            @dst << ")\n"
          end
        else
          name, value = line.split(/:/, 2)

          value.gsub!(/([^\\])\\n/, "\\1\n")
          value.gsub!(/\\\\/, "\\")

          message[name.to_sym] = value
        end
      end
    end

  end

end

# vim: set ts=4 sw=2 et:
